 addonName, addon = ...
--[[The addonName is the name by which WoW is referring to the AddOn, based on the capitalization of the AddOn's .toc file.
	The addonTable is a private table which is sent to every file in a particular AddOn (but is distinct from the table sent to other AddOns). 
	The game maintains no global registry of these tables, so it's garbage collected if not used, and an AddOn must grab its own reference to use it later.
--]]	

-- The saved variables
RareCallerDB = {};
--Learned rares
RareCallerDB.Mobs = {
	--[[
	[rareID] = { npcName = "Belgork", mapAreaID=945, coords={49.1, 72.3} },
	--]]
	--[88779] = { npcName = "Benjamin Brode", mapAreaID = 971, coords = {45,47}, lastSeen = nil, health = nil, announceNeeded = false, lastAnnounce = 0, killTime = KILLTIME_Alive, killAnnounced = 0,  lastRequest = 0 }, -- Benjamin Brode
}		
-- The global options stored cross sessions
RareCallerDB.Options = {
	RespondToRequests = nil,
	ReportFindsToGeneral = nil,
	ShowFrame = nil,
	HideInCombat = nil,
	ShowAllOnMouseOver = nil,
	SetTomTomWaypoints = nil,
	RareOrderType = nil,
	TrackingType = nil,
	TrackingRares = {},
	NpcWhiteList = { 
		[92465] = true,
		[92495] = true, --Soulslicer
	}
}

local _chatThrottle = {
	lastChatTime = 0,
	chatCount = 0 
}

local KILLTIME_Alive = -1;


-- Main refreshing and utility frame
local mainFrame  = CreateFrame("Frame", nil, UIParent)


local function NpcIDToName(npcID)
	local name;
	
	local mobInfo = RareCallerDB.Mobs[npcID];
	if (mobInfo ~= nil) then
		name = mobInfo.npcName;
		if name ~= nil then
			return name;
		end;
	end;

	--Check NPCScan's database
	local npcScan = _G["_NPCScan"];
	if (npcScan) then	
		if npcScan.NPC_ID_TO_NAME then
			name = npcScan.NPC_ID_TO_NAME[npcID];
			if (name ~= nil) then
				return name;
			end;
		end;
	end;
	
	--If we still don't know, then return the number
	return tostring(npcID);
end;

local function SameText(s1, s2)
	if s1 == nil then		
		s1 = "";
	elseif type(s1) ~= "string" then
		s1 = tostring(s1);
	end;
	if s2 == nil then		
		s2 = "";
	elseif type(s2) ~= "string" then
		s2 = tostring(s2);
	end;
	

	if string.upper(s1 or "") == string.upper(s2 or "") then	
		return true;
	else
		return false;
	end;
end;

local function NpcIsRare(npcID)
	if (npcID == nil) then	
		return false;
	end;
	
	--if (npcID == 88779) then --Benjamin Brode
	--	return true;
	--end;
	
	local mobInfo = RareCallerDB.Mobs[npcID];
	if (mobInfo == nil) then		
		return false;
	end;
	
	return true;
end;

-- The default options
local OptionsDefault = {
	RespondToRequests = true,
	ReportFindsToGeneral = true,
	ShowFrame = false,
	HideInCombat = true,
	ShowAllOnMouseOver = true,
	SetTomTomWaypoints = true,
	RareOrderType = 1, -- 1 = Name, 2 = Time
	TrackingType = 4, -- 1 = Current zone, 4=world
	TrackingRares = {}
}
local L = addon.L;

-- local text_timer, text_throttle = 0, 10
local textRefreshThrottle = 5; -- Refreshing every 5 secs
local textRefreshCurrent  = 0;
local textCanRefresh = true;
local channelJoinDelay = 15 -- Delay joining and syncing 5s after load, prevents our channel to become one of the first
local channelJoinCurrent = 0;
local needSync = false;
local syncChanNamePrefix = "RCALLER";
local addonSettingsLoaded = false
local i;

local Level = {
	None = 0;
	Critical = 1,
	Error = 2,
	Warning = 3,
	Information = 4,
	Verbose = 5,
}
addon.debugLevel = Level.Warning;
	--[[
			1: Critical
			2: Error
			3: Warning
			4: Information
			5: Verbose
	--]]

function addon:LogFmt(level, msg, ...)
	--print(string.format("LogFmt. level: %s. addon.debugLevel: %s", level, tostring(addon.debugLevel)));
	if ((level or 0) == 0) then	
		return;
	end;
	if (level > (addon.debugLevel or 0)) then
		return;
	end;
	local header = "";
	local footer = "";
	
	if (level == 5) then
		header = GRAY_FONT_COLOR_CODE; --Constants.luaor
		footer = FONT_COLOR_CODE_CLOSE; 
	elseif (level == 4) then
		header = "";
		footer = "";
	elseif (level == 3) then	
		header = ORANGE_FONT_COLOR_CODE.."Warning: "..FONT_COLOR_CODE_CLOSE;
		footer = "";
	elseif (level == 2) then
		header = RED_FONT_COLOR_CODE.."Error: "..FONT_COLOR_CODE_CLOSE;
		footer = "";
	elseif (level == 1) then
		header = RED_FONT_COLOR_CODE.."Critical: ";
		footer = FONT_COLOR_CODE_CLOSE;
	else
		header = "";
		footer = "";
	end;
	addon:Printf(header..msg..footer, ...);
end

function TimeToSystemTime(value, convertFromUtc)

	local now;
	
	if convertFromUtc == nil then	
		convertFromUtc = true;
	end;
	if convertFromUtc then	
		now = date("*t", value)
	else
		now = date("!*t", value)
	end;
		
	return now.year, now.month, now.day, now.hour, now.min, now.sec;

	--[[
	local secHour = 60*60; --seconds per hour
	local secDay = 24*secHour; --seconds per day
	local secYear = 365*secDay; --seconds per year
	local secQuadYear = 4*secYear + secDay;
	local daysPerMonthRegular = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	local daysPerMonthLeap =    { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	
	local t1 = value;

	local chunks;
	
	local year, month, day, hour, minute, second;

	
	year = 1970;
	
	--calculate number of whole four-year chunks (1 leap year + 3 not)
	chunks = math.floor(t1 / secQuadYear);
	year = year + chunks*4;
	t1 = t1 - chunks*secQuadYear;

	--remaining whole year chunks
	chunks = math.floor(t1 / secYear);
	year = year + chunks;
	t1 = t1 - chunks*secYear;

	--number of whole day chunks
	chunks = math.floor(t1 / secDay);
	day = chunks;
	t1 = t1 - chunks*secDay;
	
	--TODO: convert days to months + days
	local daysPerMonth;
	if (year % 4) == 0 then	
		daysPerMonth = daysPermOnthLeap
	else
		daysPerMonth = daysPerMonthRegular
	end;

	month = 1;
	while (day > daysPerMonth[month]) do
		day = day - daysPerMonth[month];
		month = month+1;
	end;
		
	--number of whole hour chunks
	chunks = math.floor(t1 / secHour);
	hour = chunks;
	t1 = t1 - chunks*secHour;
	
	--number of whole minute chunks
	chunks = math.floor(t1 / 60);
	minute = chunks;
	t1 = t1 - chunks*60;
	
	--what remains in t1 is seconds
	second = t1;

	return year, month, day, hour, minute, second;
	--]]
end;

function DateToStr(value)

	local year,month,day = TimeToSystemTime(value or 0);
	
	local result = L["Locale_SShortDate"] or "M/d/yyyy";
	
	--MM -> 08
	result = string.gsub(result, "MM", string.format("%.2d", month));
	--M -> 8
	result = string.gsub(result, "M", string.format("%d", month));
	
	--dd -> 01
	result = string.gsub(result, "dd", string.format("%.2d", day));
	--d -> 1
	result = string.gsub(result, "d", string.format("%d", day));
	
	--yyyy -> 2003
	result = string.gsub(result, "yyyy", string.format("%d", year));
	--yyy -> 2003
	result = string.gsub(result, "yyy", string.format("%d", year));
	--yy -> 03
	result = string.gsub(result, "yy", string.format("%d", year % 100));
	--y -> 3
	result = string.gsub(result, "yy", string.format("%d", year % 100));

	return result;
end;

function TimeToStr(value)

	local _,_,_,hour,minute,second = TimeToSystemTime(value or 0);
	local designator;

	local result = L["Locale_STimeFormat"] or "h:mm:ss tt";

	--HH -> 14
	result = string.gsub(result, "HH", string.format("%.2d", hour));
	--H -> 14
	result = string.gsub(result, "H", string.format("%d", hour));
	
	local ampmHour = hour % 12;
	if ampmHour == 0 then 
		ampmHour = 12;
	end;
	--hh -> 08
	result = string.gsub(result, "hh", string.format("%.2d", ampmhour));
	--h -> 8
	result = string.gsub(result, "h", string.format("%d", ampmHour));

	--mm -> 09
	result = string.gsub(result, "mm", string.format("%.2d", minute));
	--m -> 9
	result = string.gsub(result, "m", string.format("%d", minute));

	--ss -> 03
	result = string.gsub(result, "ss", string.format("%.2d", second));
	--s -> 9
	result = string.gsub(result, "s", string.format("%d", second));

	--tt -> AM/PM
	if hour < 12 then	
		designator = L["Locale_S1159"] or "АΜ"
	else
		designator = L["Locale_S2359"] or "PM"
	end;
	result = string.gsub(result, "tt", designator);
	
	return result;
end;

function DateTimeToStr(value)

	return DateToStr(value).." "..TimeToStr(value);
end;

function TimeToMinutesAgo(t1)
	if ((t1 or 0) == 0) then
		return "n/a";
	end;
	
	if (t1 == KILLTIME_Alive) then
		return "alive (-1)";
	end;

	--print(string.format("t1: %d", t1));
	--print(string.format("time(): %d", time()));
	
	local d = (time() - t1);
	
	--[[
	--print(string.format("d: %d", d));
	d = math.floor(d / 60);
	--print(string.format("d: %d", d));
	
	return string.format("%d min", d);
	--]]

	return SecondsToTime(d);
end;

function addon:RareTimer(rare)

	if (rare == nil) or (rare == "") then
		addon:Printf("RareTimer: Specify either npcID or partial name (e.g. /rareCaller RareTimer huo)");
		return;
	end;
	
	rare = string.lower(rare);
		
	local sNpcID, mobInfo;
	for sNpcID, mobInfo in pairs(RareCallerDB.Mobs) do
		--addon:Printf("npcName: %s (%s)", mobInfo.npcName or "", sNpcID);
		if SameText(sNpcID, rare) or (string.find(string.lower(mobInfo.npcName), rare) ~= nil) then
			addon:Printf("Last seen %s (%s) %s (%d)", mobInfo.npcName or "", sNpcID, DateTimeToStr(mobInfo.lastSeen), tostring(mobInfo.lastSeen));
			break;
		end
	end
end;

function addon:SelfTest()

	local year, month, day, hour, minute, second = TimeToSystemTime(time());
	
	addon:Printf("%d %d %d %d %d %d", year, month, day, hour, minute, second);
	
	local s = DateToStr(time());
	addon:Printf("The date is: %s", s);
	
	s = TimeToStr(time());
	addon:Printf("The time is: %s", s);

	s = DateTimeToStr(time());
	addon:Printf("The datetime is: %s", s);
end;

function addon:ToggleAnnouncements()
	if (RareCallerDB.Options.ReportFindsToGeneral) then	
		RareCallerDB.Options.ReportFindsToGeneral = false;
		addon:Printf(L["Announcements to general: disabled"]);
	else
		RareCallerDB.Options.ReportFindsToGeneral = true;
		addon:Printf(L["Announcements to general: enabled"]);
	end;
end;

function addon:ListRares(search)
	local sNpcID, mobInfo;
	local coords;

	if (search == nil) or (search == "") then	
		search = nil;
	else
		addon:Printf("Listing rares matching \"%s\"", tostring(search));
		search = string.lower(search);
	end;

	for sNpcID, mobInfo in pairs(RareCallerDB.Mobs) do

		if mobInfo.coords == nil then
			coords = "nil";
		elseif type(mobInfo.coords) ~= "table" then	
			RareCallerDB.Mobs[sNpcID].coords = nil;
			coords = "nil";
		elseif (#mobInfo.coords ~= 2) then
			RareCallerDB.Mobs[sNpcID].coords = nil;
			coords = "nil";
		else
			coords = string.format("%.1f,%.1f", RareCallerDB.Mobs[sNpcID].coords[1] or 0, RareCallerDB.Mobs[sNpcID].coords[2] or 0);
		end;
		
		if (search == nil) or (sNpcID == search) or (string.find(string.lower(mobInfo.npcName or ""), search) ~= nil) then
			addon:Printf("%s (%d): (@%s in %s) health=%s, lastSeen=%s", 
					mobInfo.npcName, 
					sNpcID,
					coords,
					tostring(mobInfo.mapAreaID), 
					tostring(mobInfo.health),
					tostring(mobInfo.lastSeen) --TimeToMinutesAgo(mobInfo.lastSeen)
			);
		end;
	end;
end;

-- Main Frame Design:
-- |-------- MainFrame -------------------------------------------|
-- ||------- NPC Name -------------------------|----  Killed ----||
-- || Some NPC Name                            |       1min      ||
-- || Some NPC Name                            |      15min      ||
-- || Some NPC Name                            |       1min      ||
-- || Some NPC Name                            |       1min      ||
-- || Some NPC Name                            |       1min      ||
-- || Some NPC Name                            |       1min      ||
-- || Some NPC Name                            |       1min      ||
-- ||------------------------------------------------------------||
-- |--------------------------------------------------------------|

local rareFrame = CreateFrame("Frame", "rareFrame", UIParent)
  rareFrame:SetWidth(261)
  rareFrame:SetHeight(34)
  rareFrame:SetPoint("LEFT", 300, 100)
  rareFrame.texture = rareFrame:CreateTexture(nil, "BACKGROUND")
  rareFrame.texture:SetTexture(0,0,0,0.4)
  rareFrame.texture:SetAllPoints(rareFrame)

  rareFrame.npcFrame = CreateFrame("Frame", "rareFrame.npcFrame", rareFrame)
  rareFrame.npcFrame:SetWidth(180)
  rareFrame.npcFrame:SetHeight(rareFrame:GetHeight()-10)
  rareFrame.npcFrame:SetPoint("TOPLEFT", rareFrame, 5, -5)
  rareFrame.npcFrame.texture = rareFrame.npcFrame:CreateTexture(nil, "BACKGROUND")
  rareFrame.npcFrame.texture:SetTexture(0,0,0,0.5)
  rareFrame.npcFrame.texture:SetAllPoints(rareFrame.npcFrame)

  rareFrame.timeFrame = CreateFrame("Frame", "rareFrame.timeFrame", rareFrame)
  rareFrame.timeFrame:SetWidth(70)
  rareFrame.timeFrame:SetHeight(rareFrame:GetHeight()-10)
  rareFrame.timeFrame:SetPoint("TOPLEFT", rareFrame.npcFrame, "TOPRIGHT", 1, 0)
  rareFrame.timeFrame.texture = rareFrame.timeFrame:CreateTexture(nil, "BACKGROUND")
  rareFrame.timeFrame.texture:SetTexture(0,0,0,0.5)
  rareFrame.timeFrame.texture:SetAllPoints(rareFrame.timeFrame)

  rareFrame.npcFrame.text = {}
  -- Header
  rareFrame.npcFrame.text[0] = rareFrame.npcFrame:CreateFontString("rareFrame.npcFrame.text[0]", nil, "GameFontNormal")
  rareFrame.npcFrame.text[0]:SetPoint("TOP", "rareFrame.npcFrame", 2, -2)
  rareFrame.npcFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12)
  rareFrame.npcFrame.text[0]:SetTextColor(1, 0.7, 0, 1)
  rareFrame.npcFrame.text[0]:SetText(L["Name"])
  rareFrame.npcFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12,"OUTLINE")

  rareFrame.timeFrame.text = {}
  -- Header
  rareFrame.timeFrame.text[0] = rareFrame.timeFrame:CreateFontString("rareFrame.timeFrame.text[0]", nil, "GameFontNormal")
  rareFrame.timeFrame.text[0]:SetPoint("TOP", "rareFrame.timeFrame", 2, -2)
  rareFrame.timeFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12)
  rareFrame.timeFrame.text[0]:SetTextColor(1, 0.7, 0, 1)
  rareFrame.timeFrame.text[0]:SetText(L["Status"])
  rareFrame.timeFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12,"OUTLINE")

local syncRareIte;
local responseFormater = CreateFrame("Button");
responseFormater:Hide();

local function DragStart()
	textCanRefresh = false
	mainFrame.updateFrame:SetScript("OnUpdate", nil)
	rareFrame:StartMoving()
end

local function DragStop()
	rareFrame:StopMovingOrSizing()
	mainFrame.updateFrame:SetScript("OnUpdate", OverallUpdater)
	textCanRefresh = true
end

--Fix missing RotateLeft operation
if (bit.rol == nil) then
	bit.rol = function(x, n)
		local result = bit.lshift(x, n) or bit.rshift(x, (32-n));
		return result;
	end;
end;

--Fix missing tobit
if (bit.tobit == nil) then
	bit.tobit = function(x)
		local result = x % 0xffffffff;
		return result;
	end;
end;

--- Returns hex representation of num
local function NumberToHex(num)
    local hexstr = '0123456789abcdef'
    local s = ''
    while num > 0 do
        local mod = math.fmod(num, 16)
        s = string.sub(hexstr, mod+1, mod+1) .. s
        num = math.floor(num / 16)
    end
    if s == '' then s = '0' end
    return s
end

function Murmur3(key, seed)
	--[[
		Murmur3_32 implementation.
		Takes a string, and returns the hex representation of the resulting number.
		If you want the actual Number return value, then go ahead and get it.
		Note: this implementation is wrong. But since Lua refuses to give us access to their hashes, 
			we need our own hashing algorithm. It doesn't have to be perfect, it just has to exist.
	--]]

	if type(key) ~= "string" then
		return -1;
	end;
	
	if seed == nil then
		seed = 0;
	end;
	
	local c1 = 0xcc9e2d51;
	local c2 = 0x1b873593;
	local r1 = 15;
	local r2 = 13;
	local m = 5;
	local n = 0xe6546b64;
	
	--hash ← seed
	local hash = seed;
	
	local keyLen = string.len(key);
	--print("keyLen: "..keyLen);
	local i = 1; --Lua strings start at one (undocumented)
	local remaining = keyLen;

	--for each fourByteChunk of key
	local k;
	while (remaining >= 4) do
		k = string.byte(key, i)+string.byte(key, i+1)*256+string.byte(key, i+2)*65536+string.byte(key, i+3)*16777216; --next four bytes of string into an integer
		k = bit.tobit(k * c1);
		k = bit.rol(k, r1);
		k = bit.tobit(k * c2);
		
		hash = bit.bxor(hash, k);
		hash = bit.rol(hash, r2);
		hash = bit.tobit(hash * m + n);
		
		remaining = remaining - 4;
		i = i + 4;
	end;
	
	--with any remainingBytesInKey
	if (remaining > 0) then
		local remainingBytes = 0;
		i = 1; --Lua strings start at one
		while (remaining > 0) do
			--print("string.byte(key, "..tostring(i).."): "..tostring(string.byte(key, i)));
			remainingBytes = remainingBytes*256 +string.byte(key, i);
			--print("remainingBytes: "..remainingBytes);
			i = i+1;
			remaining = remaining -1;
		end;
		
		k = bit.tobit(remainingBytes * c1);
		--print("k*c1: "..k);
		k = bit.rol(k, r1);
		--print("k rol r1: "..k);
		k = bit.tobit(k * c2);
		--print("k*c2: "..k);
		
		--print("hash before: "..hash);
		hash = bit.bxor(hash, k);
		--print("hash after: "..hash);
	end;
	
	hash = bit.bxor(hash, keyLen);
	--print("hash after xor: "..hash);
	
	--local c3 = 2246822507; --;
	hash = bit.bxor(hash, bit.rshift(hash, 16));
	--print("work: "..hash);
	hash = bit.tobit(hash * 0x85ebca6b);
	--print("you: "..hash);
	hash = bit.bxor(hash, bit.rshift(hash, 13));
	--print("stupid: "..hash);
	hash = bit.tobit(hash * 0xc2b2ae35);
	--print("fucking: "..hash);
	hash = bit.bxor(hash, bit.rshift(hash, 16));
	--print("cunt: "..hash);
		
	return NumberToHex(hash);	
end

local function UnitGuidToNpcID(unitGuid)
	--[[
		Converts a UnitGUID to an npcID
		
		UnitGUID                                            npcID
		--------------------------------------------------  -----
		"Creature-0-67-1464-219-93057-00000E0064"           93057 (e.g. Grannok)
		"Vehicle-0-67-1464-18-95056-00002BB94B"             95056 (e.g. Doomroller)
		"Player-67-05CD5E40"                                nil   (e.g. another player)
		"ClientActor-6-4-324"                               nil
		"Vehicle-0-68-1464-247-88695-0000171603"            nil
		"Pet-0-67-1464-17-58960-0C0121598E"                 58960 (e.g. Voidlord pet)
		
	--]]

	--print("Extracting npcID from '"..unitGuid.."'");
	local type, zero, server_id, instance_id, zone_uid, npc_id, spawn_uid = strsplit("-", unitGuid);

	--Doomroller is a "Vehicle", but still has an NpcID
	--if type ~= "Creature" then
	--	return 0;
	--end

	--print("npcID: "..npc_id);
	local npcId = tonumber(npc_id);

	--print("'"..unitGuid.."' --> "..npcId);
	return npcId;
end

local function GetMobInfoFromNpcID(rareID)
	local mobInfo = RareCallerDB.Mobs[rareID];
	
	if (mobInfo == nil) then
		--we need to know who/where this rare id. See if NPCScan can tell us anything about it
		if NpcIsRare(rareID) then
			RareCallerDB.Mobs[rareID] = {};
			mobInfo = RareCallerDB.Mobs[rareID];
		end;
	end;
	
	return mobInfo;
end;

-- Converts the mobs index into a string value
local function GetRareIdString(rareID)
	return tostring(rareID);
end

-- Gets the channel id of a named custom channel
local function GetChannelID(channelName)
	local channelCount = GetNumDisplayChannels()

	for i = 1, channelCount do
		--SetSelectedDisplayChannel(i)
		local ciName, _, _, _, _, _, _, _, _ = GetChannelDisplayInfo(i)
		if (ciName == channelName) then
			return i
		end
	end
end

-- Gets the owner of a given channel
local function GetChannelOwner(chanID)
  local _, _, _, _, memberCount, _, _, _, _ = GetChannelDisplayInfo(chanID)

  for i = 1, memberCount do
    local name, owner, _, _, _, _ = GetChannelRosterInfo(chanID, i)
    if (owner) then
      return name
    end
  end
end

-- Joins the syncing channel
local function JoinSyncChannels(self, elapsed)
	channelJoinCurrent = channelJoinCurrent + elapsed
	
	if (channelJoinCurrent > channelJoinDelay) then
		-- RareCaller sync channel
		if (IsAddonMessagePrefixRegistered(syncChanNamePrefix) == false) then
			RegisterAddonMessagePrefix(syncChanNamePrefix)
		end

		-- Now find our syncing channel
		local syncChanID = GetChannelID(syncChanNamePrefix)
		if (syncChanID == nil) then
			JoinChannelByName(syncChanNamePrefix)
		end
		needSync = true

		channelJoinCurrent = 0
	end
end

-- Leaves the syncing channels when you leave the zone
local function LeaveSyncChannels()
	if (GetChannelID(syncChanNamePrefix) ~= nil) then
		LeaveChannelByName(syncChanNamePrefix)
	end
end

-- Determines if a rare should be displayed
local function ShouldTrackRare(rareID)
	--print("should track "..rareID.."?");

	if (rareID == nil) then
		local s = "Attempt to pass rareID of nil";
		--print("ShouldTrackRare: "..s);
		--addon:LogFmt(2, "	ShouldTrackRare: %s", s);
		return false, s; 
	end;

	if (rareID == 0)  then
		local s = "Attempt to pass rareID of zero";
		--addon:LogFmt(Level.Warning, "	ShouldTrackRare: %s", s);
		return false, s; 
	end;
	
	local mobInfo = RareCallerDB.Mobs[rareID];
	if mobInfo == nil then
		--print(string.format("   ShouldTrackRare(%d): No - Rare not found", rareID));
		--addon:LogFmt(Level.Verbose, "   ShouldTrackRare(%d): %s", rareID, "No. Rare not found");
		return false;
	end;

	--[[
	[88779] = { 
		mapAreaID = 971, 
		lastAnnounce = 0, 
		lastSeen = 372348734,
		heath = 100, 
		announceNeeded = false,
		killTime = KILLTIME_Alive, 
		killAnnounced = 0, 
		npcName = (L["88779"]), 
		minSpawn = 1800, 
		maxSpawn = 3600, 
		coordsX = 45,   
		coordsY = 47,    
		lastRequest = 0 
	}
	--]]

	if (mobInfo.mapAreaID ~= nil) and (RareCallerDB.Options.TrackingType ~= 4) then
		local currentAreaID = GetCurrentMapAreaID();
		if (mobInfo.mapAreaID ~= currentAreaID) then
			local s = string.format("No - wrong zone (mob: %d, current: %d)", mobInfo.mapAreaID, currentAreaID);
			--print(s);
			--addon:LogFmt(Level.Verbose, "   ShouldTrackRare(%d): %s", rareID, s);
			return false, s;
		end;
	end;
	
	--[[
	--Check if we're in the wrong zone
	if (RareCallerDB.Mobs[rareID].mapAreaID == currentAreaID) then
		if (RareCallerDB.Options.TrackingType == 1) then --All
			local questID = RareCallerDB.Mobs[rareID].questId;
			if questID == nil then
				if vignetteMobs[rareID] ~= nil then
					questID = vignetteMobs[rareID].questId
				end
			end
			if questID ~= nil then
				shouldTrack = not IsQuestFlaggedCompleted(questID)
			else	
				shouldTrack = false
			end
		elseif (RareCallerDB.Options.TrackingType == 2) then
			--print("tracing mode: selected");
			for i = 1, #RareCallerDB.Options.TrackingRares do
				if (RareCallerDB.Options.TrackingRares[i] == rareID) then
					shouldTrack = true
					break
				end
			end
		elseif (RareCallerDB.Options.TrackingType == 3) then
			--print("tracing mode: near");
			if (RareCallerDB.Mobs[rareID].killTime ~= nil) then
				local timeSinceKill = (time() - RareCallerDB.Mobs[rareID].killTime)
				if ((timeSinceKill < RareCallerDB.Mobs[rareID].maxSpawn) and
						(timeSinceKill > RareCallerDB.Mobs[rareID].minSpawn)) then
					shouldTrack = true
				end
			end
		else
			shouldTrack = true
		end
	end
	--]]

	--print(string.format("   ShouldTrackRare(%d): %s", rareID, tostring(shouldTrack)));

	if (mobInfo.lastSeen == nil) then
		local s = "lastSeen is nil";
		return false, s;
	end;
	if (mobInfo.health == nil) then	
		local s = "health is nil";
		return false, s;
	end;

	local lastSeenSecondsAgo = (time() - (mobInfo.lastSeen or 0));
	
	if (mobInfo.health <= 0) then
		--Dead
		if (lastSeenSecondsAgo > 60) then
			--Dead and buried
			local s = "Mob is dead, but died more than 60 s ago";
			return false, s;
		end;

		--Dead, but recently
		return true, "Mob is dead (died less than 60 seconds ago)";
	elseif (mobInfo.health > 0) then
		--alive for more than 15 minutes? We must have missed their death
		if (lastSeenSecondsAgo > (60*3)) then
			--Alive, but we've not seen in a while
			return false, "Mob is alive, but was last seen more than 3 min ago";
		end;
		
		return true, "Mob is alive (and last seen less than 3 minutes ago)";
	end;
end

-- Determines what colour the time portion should be
local function DetermineTimeColour(rareID, timeText)
	local mobInfo = RareCallerDB.Mobs[rareID];
	
	if (mobInfo ~= nil) and (mobInfo.lastSeen ~= nil) then
		local timeSinceSeen = (time() - mobInfo.lastSeen)
		
		local minSpawn = mobInfo.minSpawn or 30; --minutes
		local maxSpawn = mobInfo.maxSpawn or 60; --minutes
		local mediumSpawn = (minSpawn + maxSpawn) / 2;

		if (mobInfo.health > 0) then
			--Alive
			if timeSinceSeen <= 60 then
				timeText:SetTextColor(1, 0.8, 0, 1) -- Normal
			else
				--Alive more than a minute. Outdated?
				timeText:SetTextColor(1, 0.5, 0.5, 0.5); --GRAY_FONT_COLOR
			end;
		else
			--Dead
			if timeSinceSeen <= 60 then
				--Dead less than 1 minute
				timeText:SetTextColor(1, 0.1, 0.1, 0.1) --RED_FONT_COLOR
			else
				--Dead mrore than 1 minute
				timeText:SetTextColor(1, 0.5, 0.5, 0.5); --GRAY_FONT_COLOR
			end;
		end;
	else
		timeText:SetTextColor(0, 0.8, 1, 1) -- Green
	end
end

-- Refreshes the main text window
local function RefreshText(reason)
	--print(string.format("RefreshText(%s)", reason or "nil"));
	
	if (addonSettingsLoaded == false) then
		return;
	end;
	
	if (rareFrame.npcFrame == nil) then
		return;
	end;
	
	if (rareFrame.npcFrame.text == nil) then
		return;
	end;
	
	-- First clear the frame
	for textIndex = 1, #rareFrame.npcFrame.text do
		rareFrame.npcFrame.text[textIndex]:SetText("");
		rareFrame.timeFrame.text[textIndex]:SetText("");
	end
	rareFrame.npcFrame.text = {};
	rareFrame.timeFrame.text = {};

	-- Redo the headers
	rareFrame.npcFrame.text[0] = rareFrame.npcFrame:CreateFontString("rareFrame.npcFrame.text[0]", nil, "GameFontNormal");
	rareFrame.npcFrame.text[0]:SetPoint("TOP", "rareFrame.npcFrame", 2, -2);
	rareFrame.npcFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12);
	rareFrame.npcFrame.text[0]:SetTextColor(1, 0.7, 0, 1);
	rareFrame.npcFrame.text[0]:SetText(L["Name"]);
	rareFrame.npcFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12,"OUTLINE");

	rareFrame.timeFrame.text[0] = rareFrame.timeFrame:CreateFontString("rareFrame.timeFrame.text[0]", nil, "GameFontNormal");
	rareFrame.timeFrame.text[0]:SetPoint("TOP", "rareFrame.timeFrame", 2, -2);
	rareFrame.timeFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12);
	rareFrame.timeFrame.text[0]:SetTextColor(1, 0.7, 0, 1);
	rareFrame.timeFrame.text[0]:SetText(L["Status"]);
	rareFrame.timeFrame.text[0]:SetFont("Fonts\\ARIALN.TTF", 12,"OUTLINE");

	-- Then repopulate them
	-- Loop through our rare mobs, check if we have a name and kill time, then search for it in npcFrame
	-- if it is there, update the time, otherwise add it
	local textIndex = 0
	local rareIndex = 0
	local textFound = false
	
	local lastSeenSecondsAgo;
	
	local sNpcID, mobInfo;
	for sNpcID, mobInfo in pairs(RareCallerDB.Mobs) do
		local rareID = tonumber(sNpcID or "0");
		if (rareID > 0) then
			if (ShouldTrackRare(rareID)) then
				local newIndex = #rareFrame.npcFrame.text + 1;
				rareFrame.npcFrame.text[newIndex] = rareFrame.npcFrame:CreateFontString("rareFrame.npcFrame.text["..newIndex.."]", nil, "GameFontNormal");
				rareFrame.npcFrame.text[newIndex]:SetPoint("TOP", "rareFrame.npcFrame", 2, -0 + -14.* newIndex);
				rareFrame.npcFrame.text[newIndex]:SetFont("Fonts\\ARIALN.TTF", 12);
				rareFrame.npcFrame.text[newIndex]:SetTextColor(1, 0.8, 0, 1);
				rareFrame.npcFrame.text[newIndex]:SetText(mobInfo.npcName or sNpcID);
				rareFrame.npcFrame.text[newIndex]:SetJustifyH("LEFT");

				rareFrame.timeFrame.text[newIndex] = rareFrame.timeFrame:CreateFontString("rareFrame.timeFrame.text["..newIndex.."]", nil, "GameFontNormal");
				rareFrame.timeFrame.text[newIndex]:SetPoint("TOP", "rareFrame.timeFrame", 3, 0 + -14.* newIndex);
				rareFrame.timeFrame.text[newIndex]:SetFont("Fonts\\ARIALN.TTF", 12);
				if ((mobInfo.health or 0) > 0) then
					rareFrame.timeFrame.text[newIndex]:SetText(tostring(mobInfo.health).."%"); --L["Alive"]);
				elseif (mobInfo.lastSeen) then
					lastSeenSecondsAgo = time() - mobInfo.lastSeen;
					if (lastSeenSecondsAgo <= 60 ) then
						rareFrame.timeFrame.text[newIndex]:SetText(L["Dead"]);
					else
						rareFrame.timeFrame.text[newIndex]:SetText(SecondsToTime(lastSeenSecondsAgo, true))
					end;
				else
					rareFrame.timeFrame.text[newIndex]:SetText(L["Dead"]);
				end;
				DetermineTimeColour(rareIndex, rareFrame.timeFrame.text[newIndex]);
			end;
		end
	end
	rareFrame.npcFrame:SetHeight(25 +  #rareFrame.npcFrame.text *14);
	rareFrame.timeFrame:SetHeight(25 +  #rareFrame.npcFrame.text *14);
	rareFrame:SetHeight(35 +  #rareFrame.npcFrame.text *14);
	
	if #rareFrame.npcFrame.text > 0 then
		rareFrame:Show();
	else
		rareFrame:Hide();
	end;
end

-- General OnEvent handler (Refresher)
local function OverallUpdater(self, elapsed)
	textRefreshCurrent = textRefreshCurrent + elapsed;
	
	if (textRefreshCurrent >= textRefreshThrottle) then
		if (textCanRefresh == true) then
			--Refresh the text frames
			RefreshText("OverallUpdater")
		end;

		textRefreshCurrent = 0
	end
end

-- Sends a message to either /1 or our sync channel
local function SendGeneralMessage(msg, isSync)
	--print(string.format("SendGeneralMessage(\"%s\", %s)", tostring(msg), tostring(isSync)));

	if (msg == nil) then
		return;
	end;
	
	if (isSync == false) then
		--Send to General channel
		
		local index;
		index = GetChannelName("General"); -- Find General Channel for user
		--index = GetChannelName("RareCallerTest"); --our own spamming test channel
		if (index == 0) then
			--print("Could not locate \"General\" channel. Assuming channel number 1");
			index = 1;
		end;
		
		if (index~=nil) then
			-- For some reason the SendChatMessage does not like the output from SecondsToTime. This is a work around.
			msg = responseFormater:GetText(responseFormater:SetText(msg))

			--print("sending to channel "..index);
			SendChatMessage(msg , "CHANNEL", nil, index)
			-- SendChatMessage(msg , "CHANNEL", nil, 4) -- Testing
		end
	else
		local syncChanID = GetChannelName(syncChanNamePrefix);
		--print(string.format("SendChatMessage(\"%s\", \"CHANNNEL\", nil, %s)", msg, tostring(syncChanID)));
		if (syncChanID ~= nil) then
			SendChatMessage(msg, "CHANNEL", nil, syncChanID);
		end
	end
end

-- Creates and sends the death message to the sync and general channels
local function SendDeathMessage(rareID)
	--print(string.format("SendDeathMessage(%s)", tostring(rareID)));
	if (RareCallerDB.Mobs[rareID] == nil) then
		return;
	end;
	
	local deathMsg;
	
	--[[
		Announce to other addons that the rare has died
		
			[RCALLER]{rareID}_kill_{time}
			
		e.g.
			[RCALLER]40283_kill_3783898389
	--]]
	deathMsg = "[".. syncChanNamePrefix .. "]" .. GetRareIdString(rareID) .. "_kill_" .. tostring(RareCallerDB.Mobs[rareID].lastSeen);
	SendGeneralMessage(deathMsg, true);

	
	if (RareCallerDB.Options.ReportFindsToGeneral) then
		-- Kill announcement throttle. Don't announce if it was announced less than 2 minutes ago
		-- The reason we do this is because someone else could have announced it for us.
		if ((time() - (RareCallerDB.Mobs[rareID].killAnnounced) or 0) < 120) then
			return;
		end;
		
		RareCallerDB.Mobs[rareID].killAnnounced = time();

		-- Announce that we are about to announce the death
		deathMsg = "[".. syncChanNamePrefix .. "]" .. GetRareIdString(rareID) .. "_killannounced_" .. tostring(RareCallerDB.Mobs[rareID].killAnnounced)
		SendGeneralMessage(deathMsg, true)

		-- And now announce the death
		local n = random(5);
		local fmt = L["DeathAnnouncement"..tostring(n)];
		deathMsg = string.format(fmt, RareCallerDB.Mobs[rareID].npcName);
		SendGeneralMessage(deathMsg, false)
	end
end

-- Creates and sends the target messages and sync
local function SendTargetMessage(rareID)
	--print(string.format("SendTargetMessage(%s)", tostring(rareID)));

	local mobInfo = RareCallerDB.Mobs[rareID];
	if (mobInfo == nil) then
		return;
	end;
	
	-- Only announce once every 30 seconds
	if time() < ((mobInfo.lastAnnounce or 0) + 30) then
		return;
	end;
	
	local name = mobInfo.npcName or UnitName("target");
	local x, y = GetPlayerMapPosition("player");
	local health = math.floor(UnitHealth("target")*100/UnitHealthMax("target")); --as a percentage (0..100)

	mobInfo.lastAnnounce = time();

	--Announce to other addons that we've spotted the rare
	local targetMessage;
	--targetMessage = "[".. syncChanNamePrefix .. "]" ..GetRareIdString(rareID) .. "_target_" .. RareCallerDB.Mobs[rareID].lastAnnounce

	--[RCALLER]{npcID}_s_{time}_{healthPercentage}_{coords}_{mapAreaID}_{npcName}
	local targetMessage = string.format("[%s]%d_s_%d_%d_%d_%d_%s",
			syncChanNamePrefix,
			rareID,
			mobInfo.lastSeen,
			mobInfo.health,
			mobInfo.coords[1]*10000+mobInfo.coords[2]*10,
			mobInfo.mapAreaID,
			mobInfo.npcName);

	SendGeneralMessage(targetMessage, true);

	if (RareCallerDB.Options.ReportFindsToGeneral == true) then
		--print("   Report to general enabled");
		
		--Announce that we're about to announce the rare
		targetMessage = "[".. syncChanNamePrefix .. "]" .. GetRareIdString(rareID) .. "_aliveannounced_" .. tostring(RareCallerDB.Mobs[rareID].lastAnnounce)
		SendGeneralMessage(targetMessage, true)

		--[[
			And now announce the rare in general
		--]]
		local n = random(5);
		local fmt;
		if n == 1 then
			--Garrick Padfoot (100%) Coordinates: 57,37
			--targetMessage = mobInfo.npcName .." (".. health .."%) "..(L["Coordinates"]).." "..math.floor(x*100)..","..math.floor(y*100);
			--fmt = "%s (%d%%) Coordinates: %d,%d";
			fmt = L["Announcement1"];
			targetMessage = string.format(fmt,
					mobInfo.npcName,
					health,
					math.floor(x*100), math.floor(y*100)
			);
		else
			--Garrick Padfoot up 57,80 (100%)
			--Garrick Padfoot up at 57,80 (100%)
			--Garrick Padfoot 57,80 (100%)
			--Garrick Padfoot is up 57,80 (75%)
			fmt = L["Announcement"..tostring(n)];
			targetMessage = string.format(fmt, 
					mobInfo.npcName, 
					math.floor(x*100), math.floor(y*100),
					health);
		end;

		SendGeneralMessage(targetMessage, false)
	end
end

-- Creates and sends the request response message and sync
local function SendRequestMessage(rareID)
	addon:LogFmt(Level.Verbose, "SendRequestMessage(\"%s\")", tostring(rareID));
	
	if (RareCallerDB.Options.RespondToRequests ~= true) then
		return;
	end;
	
	local mobInfo = RareCallerDB.Mobs[rareID];
	if (mobInfo == nil) then
		return;
	end;
	
	if (mobInfo.lastRequest ~= nil) and (mobInfo.lastRequest <= 0) then
		RareCallerDB.Mobs[rareID].lastRequest = nil;
	end;
	
	-- We throttle the number of requests per mob to once every 1 minute
	if (time() < ((mobInfo.lastRequest or 0) + 60))
			or (mobInfo.npcName == nil) 
			or (mobInfo.lastSeen == nil)
			or (mobInfo.health == nil) then
		addon:LogFmt(Level.Information, 'Not announcing NPC %s because reasons', tostring(rareID));
		return;
	end;

	local responseString;

	--First announce that we're going to announce
	RareCallerDB.Mobs[rareID].lastRequest = time();
	responseString = "[".. syncChanNamePrefix .. "]" ..GetRareIdString(rareID) .. "_request_" .. RareCallerDB.Mobs[rareID].lastRequest;
	SendGeneralMessage(responseString, true)

	--Then do the actual announce
	if (RareCallerDB.Mobs[rareID].health > 0) then
		responseString = string.format(L["ResponseAlive"], mobInfo.npcName or "", mobInfo.health or 100);
	elseif ((time() - mobInfo.lastSeen) < 60) then
		responseString = string.format(L["ResponseRecentKill"], mobInfo.npcName or "");
	else
		responseString = string.format(L["ResponseLastKilled"], mobInfo.npcName, tostring(SecondsToTime(time() - mobInfo.lastSeen, true)));
	end;
		
	SendGeneralMessage(responseString, false);
end

-- Request handler
local function ProcessRequest(message)
	addon:Printf("ProcessRequest(\"%s\")", tostring(message));

	-- Expected format = raretime <npcname / id>
	if (RareCallerDB.Options.RespondToRequests ~= true) or (message == nil) then
		return;
	end;
	
	local searchingString;

	if (strlower(string.sub(message,1,9)) == "raretime ") then
		searchingString = string.sub(message, 10, string.len(message));
	elseif (strlower(string.sub(message,1,10)) == "raretimer ") then
		searchingString = string.sub(message, 11, string.len(message));
	else
		return;
	end

	if (string.len(searchingString) < 4) then
		return;
	end;
	
	local responseString

	-- First check for ID
	--local searchID = tonumber(searchingString)
	local sNpcID, mobInfo;
	for sNpcID, mobInfo in pairs(RareCallerDB.Mobs) do
		--addon:Printf("npcName: %s (%s)", mobInfo.npcName or "", sNpcID);
		if SameText(sNpcID, searchingString) or (string.find(string.lower(mobInfo.npcName), searchingString) ~= nil) then
			addon:Printf("Last seen %s (%s) %s (%d)", mobInfo.npcName or "", sNpcID, DateTimeToStr(mobInfo.lastSeen), tostring(mobInfo.lastSeen));
			SendRequestMessage(tonumber(sNpcID));
			break;
		end
	end
end

-- Deathlog handler
local function ProcessDeathLog(name, guid)
	--print(string.format("ProcessDeathLog(\"%s\", %s)", tostring(name), tostring(guid)));
	
	local npcID = UnitGuidToNpcID(guid); --returns 0 if not an npc
	
	if (npcID == nil) or (npcID <= 0) then	
		--print("   Could not get npcID from unitGUID");
		return;
	end;
	
	--if not NpcIsRare(npcID) then	
	--	return;
	--end;

	local mobInfo = RareCallerDB.Mobs[npcID];
	if (mobInfo == nil) then
		--print(string.format("npcID %s not found in mobs table (they are not rare)", tostring(npcID)));
		return;
	end;

	RareCallerDB.Mobs[npcID].npcName = name;
	RareCallerDB.Mobs[npcID].lastSeen = time();
	RareCallerDB.Mobs[npcID].health = 0; 
	RareCallerDB.Mobs[npcID].announceNeeded = true;
	--RareCallerDB.Mobs[npcID].killTime = RareCallerDB.Mobs[rareID].lastSeen;
	SendDeathMessage(npcID);
	RefreshText("ProcessDeathLog");
end

-- Our Sync channel message handler
local function ProcessSyncChatMsg(message)
	--print(string.format("ProcessSyncChatMsg(\"%s\"", tostring(message)));
	local prefix = "[" .. syncChanNamePrefix .. "]";
	
	local dataStr = string.sub(message, string.len(prefix)+1, string.len(message));
	--[[
			[RCALLER]91093_kill_1437936113_
			[RCALLER]91093_killannounced_1437936113
			[RCALLER]91093_target_1437936113
			[RCALLER]91093_aliveannounced_1437936113
			[RCALLER]91093_request_1437936113

			[RCALLER]103_s_1437936113_100_575482_978_Garrick Padfoot
			[RCALLER]{npcID}_s_{time}_{healthPercentage}_{coords}_{mapAreaID}_{npcName}
	--]]

	local rareIDStr, syncType, syncTimeStr, health, coords, mapAreaID, npcName = strsplit("_", dataStr);
	
	if ((rareIDStr == nil) or (syncType == nil) or (syncTimeStr == nil)) then
		return;
	end;
	
	local rareID = tonumber(strtrim(rareIDStr));
	local syncTime = tonumber(strtrim(syncTimeStr));

	if ((rareID == nil) or (syncTime == nil)) then
		return;
	end;

	local mobInfo = GetMobInfoFromNpcID(rareID);

	--for most of these messages it only makes sense for us to process it if we already know about the mob (we're simply processing updates)
	--But this addon also supports communicating of new rares (todo: hmac to prevent spoofing)
	if (syncType == "s") then
		addon:LogFmt(Level.Information, "Receiving npc %d status update at %d", rareID, syncTime);
		--Add the mob if we can
		if mobInfo == nil then	
			RareCallerDB.Mobs[rareID] = {};
			
			if (RareCallerDB.Mobs[rareID].npcName == nil) and (npcName) then
				RareCallerDB.Mobs[rareID].npcName = npcName;
			end;
			if (RareCallerDB.Mobs[rareID].mapAreaID == nil) and (mapAreaID) then
				RareCallerDB.Mobs[rareID].mapAreaID = tonumber(mapAreaID);
			end;
			if (RareCallerDB.Mobs[rareID].coords == nil) and (coords) then
				local nCoords = tonumber(coords);
				if nCoords ~= nil then	
					RareCallerDB.Mobs[rareID].coords = {
							(nCoords % 1000) / 10,
							(nCoords / 1000) / 10
					};
				end;
			end;
		end;
			
		if (health) then
			local nHealth = tonumber(health);
			if (nHealth ~= nil) then
				RareCallerDB.Mobs[rareID].health = nHealth;
				RareCallerDB.Mobs[rareID].lastSeen = syncTime;
				RefreshText("Chat status update");
			end;
		end;
		
		return;
	end;
	
	if (mobInfo ~= nil) then
		if (syncType == "request") then
			mobInfo.lastRequest = syncTime;
		elseif (syncType == "kill") or (syncType == "death") then
			mobInfo.lastSeen = syncTime;
			mobInfo.health = 0;
		elseif (syncType == "target") then
			mobInfo.lastAnnounce = syncTime;
			mobInfo.lastSeen = syncTime;
			if (mobInfo.health or 0) <= 0 then
				mobInfo.health = 100;
			end;
			mobInfo.announceNeeded = false;
		elseif (syncType == "killannounced") then
			mobInfo.killAnnounced = syncTime;
		end
	end
end

-- RareCoordinator sync channel message handler
local function ProcessRCSyncChatMsg(message)
	local prefix = "[" .. syncChanNamePrefix .. "]"
	local dataStr
	if string.find(message, prefix) then
		dataStr = string.sub(message, string.len(prefix)+1, string.len(message))
	else
		dataStr = message
	end

	local rcVersion, rcRareIDStr, rcType, rcTimeStr = strsplit("_", dataStr);
	if ((rcRareIDStr ~= nil) and (rcTimeStr   ~= nil)) then
		local rcRareID = tonumber(rcRareIDStr)
		local rcTime   = tonumber(rcTimeStr)

		if ((rcVersion ~= nil) and
				(rcRareID  ~= nil) and
				(rcType    ~= nil) and
				(rcTime    ~= nil)) then
			if (rcType == "killed") then
				if (RareCallerDB.Mobs[rcRareID] ~= nil) then
					-- Only update if the kill time is less than ours
					if (RareCallerDB.Mobs[rcRareID].killTime ~= nil) then
						if (RareCallerDB.Mobs[rcRareID].killTime > rcTime) and (rcTime > 0) then
							RareCallerDB.Mobs[rcRareID].killTime = rcTime
						elseif (RareCallerDB.Mobs[rcRareID].killTime == KILLTIME_Alive) then
							RareCallerDB.Mobs[rcRareID].killTime = rcTime
						end
					else
						RareCallerDB.Mobs[rcRareID].killTime = rcTime
					end
				end
			elseif (rcType == "alive") then
				if (RareCallerDB.Mobs[rcRareID] ~= nil) then
					RareCallerDB.Mobs[rcRareID].killTime = nil
				end
			end
		end
	end
end

-- The hidden addon channel message handler
local function ProcessAddonMsg(prefix, message, channel, sender)
	if (prefix == syncChanNamePrefix)then
		if (channel == "WHISPER") then
			-- Sync request
			if (message == "Sync") then
				local sNpcID, mobInfo;
				local message;
				for sNpcID, mobInfo in pairs(RareCallerDB.Mobs) do
					if (mobInfo.lastSeen ~= nil) and (mobInfo.health ~= nil) then
						message = "[".. syncChanNamePrefix .. "]" .. GetRareIdString(mobIDs[syncRareIte]) .. "_death_" .. tostring(RareCallerDB.Mobs[mobIDs[syncRareIte]].killTime);
						SendAddonMessage(syncChanNamePrefix, message, "WHISPER", sender)
					end
				end
				-- Sync response
			else
				ProcessSyncChatMsg(message)
				-- RefreshText(self, 100)
			end
		end
	end
	--]]
end

-- Channel roster update handler (used for requesting syncs)
local function ProcessChanRosterUpdate(chanID)
	local name, _, _, _, _, _, _, _, _ = GetChannelDisplayInfo(chanID)

	if (name == syncChanNamePrefix) then
		-- Check if we need to sync
		if (needSync) then
			mainFrame:SetScript("OnUpdate", nil)
			needSync = false

			local chanOwner = GetChannelOwner(chanID)
			if (chanOwner ~= nil) then
				SendAddonMessage(syncChanNamePrefix, "Sync", "WHISPER", chanOwner)
			end
		end
	end

	if ((needSync   == false) or (needRCSync == false)) then
		mainFrame:SetScript("OnUpdate", nil)
	else
		mainFrame:SetScript("OnUpdate", JoinSyncChannels)
	end
end

-- Chat message handler for /1 and the sync channels
local function ProcessChatMsg(message, sender, language, channelString, target, flags, unknown, channelID, channelName, unknown, counter, guid)

	if (channelID == 1) then
		-- Handle request messages
		-- Request string
		if (strlower(string.sub(message,1,9)) == "raretime ") or (strlower(string.sub(message,1,10)) == "raretimer ") then
			-- Expected format =  ratime <npc name/id>
			ProcessRequest(message);
		end
	elseif (channelName == syncChanNamePrefix) then
		-- Addon Channel message
		--print("Sender: "..tostring(sender));
		if string.find(message, "%["..syncChanNamePrefix.."%]") then
			ProcessSyncChatMsg(message)
		end
	end
end

-- Zone changing event handler
local function EnableOrDisable()

	local show = not IsInInstance();

	--local mapAreaID = GetCurrentMapAreaID();
	-- MapAreaID 951 = Timeless Isles
	-- MapAreaID 945 = Tanaan Jungle
	-- MapAreaID 971 = Garrison
	--local timelessIsleAreaID = 951;
	--local tanaanJungleAreaID = 945;
	--local garrisonAreaID = 971;
	--if ((mapAreaID == 951) or (mapAreaID == 945) or (mapAreaID == 971))  then
	--	show = false;
	
	if show then
		mainFrame:RegisterEvent("CHAT_MSG_CHANNEL")
		mainFrame:RegisterEvent("CHAT_MSG_ADDON")
		mainFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
		mainFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		mainFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
		mainFrame:RegisterEvent("PLAYER_REGEN_ENABLED")

		mainFrame:SetScript("OnUpdate", JoinSyncChannels)

		rareFrame:Show()
	else
		mainFrame:UnregisterEvent("CHAT_MSG_CHANNEL")
		mainFrame:UnregisterEvent("CHAT_MSG_ADDON")
		mainFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
		mainFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		mainFrame:UnregisterEvent("PLAYER_REGEN_DISABLED")
		mainFrame:UnregisterEvent("PLAYER_REGEN_ENABLED")

		mainFrame:SetScript("OnUpdate", nil)
		LeaveSyncChannels()

		rareFrame:Hide()
		needSync = true
		needRCSync = true
	end
	
	return show;
end

local function ZoneChangedNewArea()

	local enabled = EnableOrDisable();
	
	if enabled then
		RefreshText("ZoneChangedNewArea");
	end;
end;


-- Player into combat event handler
local function HideFrameInCombat()
  if (RareCallerDB.Options.HideInCombat == true) then
    rareFrame:Hide()
  end
end

-- Player out of combat event handler
local function ShowFrameAfterCombat()
  if (RareCallerDB.Options.HideInCombat == true) and (RareCallerDB.ShowFrame) then
    rareFrame:Show()
  end
end

-- Loads the saved variables when login and set the default as needed
local function LoadSavedVariables()
	if (RareCallerDB.Options == nil) then
		RareCallerDB.Options = CopyTable(OptionsDefault)
	end

	if (RareCallerDB.Options.RespondToRequests == nil) then
		RareCallerDB.Options.RespondToRequests = OptionsDefault.RespondToRequests
	end

	if (RareCallerDB.Options.ReportFindsToGeneral == nil) then
		RareCallerDB.Options.ReportFindsToGeneral = OptionsDefault.ReportFindsToGeneral
	end

	RareCallerDB.Options.ShowFrame = RareCallerDB.Options.ShowFrame or OptionsDefault.ShowFrame;
	
	if (RareCallerDB.Options.HideInCombat == nil) then
		RareCallerDB.Options.HideInCombat = OptionsDefault.HideInCombat
	end

	if (RareCallerDB.Options.ShowAllOnMouseOver == nil) then
		RareCallerDB.Options.ShowAllOnMouseOver = OptionsDefault.ShowAllOnMouseOver
	end

	if (RareCallerDB.Options.SetTomTomWaypoints == nil) then
		RareCallerDB.Options.SetTomTomWaypoints = OptionsDefault.SetTomTomWaypoints
	end

	if (RareCallerDB.Options.RareOrderType == nil) then
		RareCallerDB.Options.RareOrderType = OptionsDefault.RareOrderType
	end

	if (RareCallerDB.Options.TrackingType == nil) then
		RareCallerDB.Options.TrackingType = OptionsDefault.TrackingType
	end
	
	if (RareCallerDB.Mobs == nil) then	
		RareCallerDB.Mobs = {};
	end;

	addonSettingsLoaded = true
end

local function GetMobInfoWithTarget(rareID)
	if rareID then
		addon:LogFmt(Level.Verbose, "GetMobInfoWithTarget(%s)", tostring(rareID));
		--print(string.format("GetMobInfoWithTarget(%s)", tostring(rareID)));
		
		--Make sure the the mob exists in our saved variables RareCallerDB.
		if RareCallerDB.Mobs == nil then		
			RareCallerDB.Mobs = {};
		end;
		if RareCallerDB.Mobs[rareID] == nil then
			RareCallerDB.Mobs[rareID] = {};
		end;
		
		local npcName = strtrim(UnitName("target") or "");
		RareCallerDB.Mobs[rareID].npcName = npcName;
			
		local mapAreaID,_ = GetCurrentMapAreaID();
		RareCallerDB.Mobs[rareID].mapAreaID = mapAreaID;
			
		local x, y = GetPlayerMapPosition("player"); --e.g. 0.4913,0.7237
		RareCallerDB.Mobs[rareID].coords = {
				math.floor(x*1000)/100, 
				math.floor(y*1000)/100 
		};

		RareCallerDB.Mobs[rareID].mapAreaID = mapAreaID;
		--print("setting target killTime to alive");
		
		RareCallerDB.Mobs[rareID].lastSeen = time();
		RareCallerDB.Mobs[rareID].health = math.floor(UnitHealth("target")*100/UnitHealthMax("target")); --as a percentage (0..100);
		RareCallerDB.Mobs[rareID].announceNeeded = true;

		RareCallerDB.Mobs[rareID].killTime = KILLTIME_Alive; -- Reset the kill time to Alive
		--print(string.format("RareCallerDB.Mobs[%s].killTime = %s", tostring(rareID), tostring(RareCallerDB.Mobs[rareID].killTime))); 
		RareCallerDB.Mobs[rareID].killAnnounced = 0;
		--print(string.format("RareCallerDB.Mobs[%s].killTime = %s", tostring(rareID), tostring(RareCallerDB.Mobs[rareID].killTime))); 
		
		return RareCallerDB.Mobs[rareID];
	else
		return {};
	end;
end;	

-- Main event handler
local function MainEvents(mainFrame, event, ...)
	if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		local _,event_type,_,_,_,_,_,guid,name = select(1, ...); --timestamp,event,hideCaster,sourceGUID,sourceName,sourceRaidFlags,destGUID,destName,destFlags,destRaidFlags

		if event_type == "UNIT_DIED" then
			ProcessDeathLog(name, guid)
		end
	elseif (event == "PLAYER_TARGET_CHANGED") then
		--This event is also fired when you un-target something. UnitGUID of your target will be nil
		local guid = UnitGUID("target");
		
		if (guid == nil) then
			return;
		end;

		local name = strtrim(UnitName("target") or "");
		local npcID = UnitGuidToNpcID(guid) or 0;
		
		--[[
			If the target is dead, check that we don't still think it's alive.
			If we do, update ourselves and possibly notify the world.
		--]]
		if UnitIsDead("target") and (RareCallerDB.Mobs[npcID] ~= nil) then
			if RareCallerDB.Mobs[npcID].health == 0 then
				--We already think he's dead
				return;
			end;
				
			local timeLastSeen = RareCallerDB.Mobs[npcID].lastSeen;
			
			addon:Printf("Updating the fact that mob %d is dead", npcID);
			RareCallerDB.Mobs[npcID].health = 0;
			RareCallerDB.Mobs[npcID].lastSeen = time();
			
			--If it hasn't been too long (30), report the death
			if (RareCallerDB.Mobs[npcID].lastSeen - (timeLastSeen or 0)) < 30 then
				ProcessDeathLog(name, guid);
			end;
			return;
		end;
		
		local classification = UnitClassification("target") or ""; --"elite", "minus", "normal", "rare", "rareelite", "worldboss"
		
		--[[
			We only care about "rareelite"
			Except that rare elite "Relgor" is "rare"
		--]]
		local reportNpc = ((classification == "rareelite") or (classification == "rare") or NpcIsRare(npcID));
		--reportNpc = reportNpc or RareCallerDB.NpcWhiteList[npcID];
		if reportNpc then
			local mobInfo = GetMobInfoWithTarget(npcID);
			
			--addon:LogFmt(Level.Verbose, "Targeting "..classification.." \""..mobInfo.npcName.."\" (npcID: "..tostring(npcID)..",UnitGUID: "..guid..")");
			SendTargetMessage(npcID);
			RefreshText("Targeted a live rare");
		--else
		--	print("Targeting "..classification.." \""..name.."\" (npcID: "..tostring(npcID)..", UnitGUID: "..guid..")");
		end
	elseif (event == "CHAT_MSG_CHANNEL") then
		ProcessChatMsg(...)
	elseif (event == "CHAT_MSG_ADDON") then
		ProcessAddonMsg(...)
	elseif (event == "PLAYER_REGEN_DISABLED") then
		HideFrameInCombat()
	elseif (event == "PLAYER_REGEN_ENABLED") then
		ShowFrameAfterCombat()
	elseif (event == "PLAYER_ENTERING_WORLD") then
		EnableOrDisable()
	elseif (event == "ZONE_CHANGED_NEW_AREA") then
		ZoneChangedNewArea()
	elseif (event == "CHANNEL_ROSTER_UPDATE") then
		ProcessChanRosterUpdate(...)
	elseif (event == "ADDON_LOADED") then
		LoadSavedVariables()
	end
end
-- === OPTION PANELS START === --

-- Generic CheckBox maker
local function CreateCheckButton(parent, checkBoxName, posX, posY, displayText)
	local checkButton = CreateFrame("CheckButton", checkBoxName, parent, "UICheckButtonTemplate");
	checkButton:SetPoint("TOPLEFT", posX, posY);
	checkButton:SetWidth(25);
	checkButton:SetHeight(25);
	getglobal(checkButton:GetName() .. 'Text'):SetText(displayText);

	return checkButton;
end

-- Generic RadionButton maker
local function CreateRadioButton(parent, radioButtonName, posX, posY, displayText)
	local radioButton = CreateFrame("CheckButton", radioButtonName, parent, "UIRadioButtonTemplate");
	radioButton:SetPoint("TOPLEFT", posX, posY);
	radioButton:SetHeight(20);
	radioButton:SetWidth(20);
	getglobal(radioButton:GetName() .. 'Text'):SetText(displayText);
	return radioButton;
end

-- === OPTION PANELS START === --
	function OptionsFrameRefreshText()
		if ((rareFrame.npcFrame.text ~= nill) and
			(rareFrame.timeFrame.text ~= nil)) then
			for textIndex = 1, #rareFrame.npcFrame.text do
				rareFrame.npcFrame.text[textIndex]:SetText("")
				rareFrame.timeFrame.text[textIndex]:SetText("")
			end
		end
		rareFrame.npcFrame.text = {}
		rareFrame.timeFrame.text = {}
		OverallUpdater(mainFrame, 10)
	end

	-- Set the options when showing the interface options menu
	function OptionsFrameSettingsLoad()
		mainFrame.optionsFrame.chkRespondToRequest:SetChecked(RareCallerDB.Options.RespondToRequests);
		mainFrame.optionsFrame.chkReportFindsToGeneral:SetChecked(RareCallerDB.Options.ReportFindsToGeneral);
		mainFrame.optionsFrame.chkHideInCombat:SetChecked(RareCallerDB.Options.HideInCombat);
		--mainFrame.optionsFrame.chkSetTomTom:SetChecked(RareCallerDB.Options.SetTomTomWaypoints);
		
		-- Tracking type
		if (RareCallerDB.Options.TrackingType == 1) then
			mainFrame.optionsFrame.rbnTrackAll:SetChecked(false);
			mainFrame.optionsFrame.rbnTrackCurrentZone:SetChecked(true);
		elseif (RareCallerDB.Options.TrackingType == 4) then
			mainFrame.optionsFrame.rbnTrackAll:SetChecked(true);
			mainFrame.optionsFrame.rbnTrackCurrentZone:SetChecked(false);
		end
	end

	-- Event handler for the tracking type radio buttons
	function SetTrackingType(sender, sourceButton, down)
		-- 1 = Current Zone, 4 = All
		if (sender:GetName() == "rbnTrackAll") then
			RareCallerDB.Options.TrackingType = 4
			rbnTrackCurrentZone:SetChecked(false)
		elseif (sender:GetName() == "rbnTrackCurrentZone") then
			RareCallerDB.Options.TrackingType = 1
			rbnTrackAll:SetChecked(false)
		end
	end

	function UpdateTrackingList(sender, sourceButton, down)
		-- If the tracking type is "Selected" then do the below, otherwise se the check back to its previous state
		if (RareCallerDB.Options.TrackingType == 2) then
			-- get the index number
			local bracketStart, _ = string.find(sender:GetName(), "%[")
			local _, bracketEnd = string.find(sender:GetName(), "%]")
			local indexStr
			if ((bracketStart ~= nil) and (bracketEnd ~= nil)) then
				indexStr = string.sub(sender:GetName(), bracketStart+1, bracketEnd-1)
			end

			local rareIndex = tonumber(indexStr)
		
			local found = false
			if (rareIndex ~= nil) then
				for i = 1, #RareCallerDB.Options.TrackingRares do
					if (RareCallerDB.Options.TrackingRares[i] == mobIDs[rareIndex]) then
						found = true
						-- If the rare is in the list, remove it if it is now unchecked
						if (not sender:GetChecked()) then
							table.remove(RareCallerDB.Options.TrackingRares, i)
						end
					end
				end

				if (found == false) then
					if (sender:GetChecked()) then
						tinsert(RareCallerDB.Options.TrackingRares, mobIDs[rareIndex])
					end
				end
			end
		else
			sender:SetChecked(not sender:GetChecked())
		end
	end

	-- Main Panel
	mainFrame.optionsFrame = CreateFrame("ScrollFrame", "mainFrame.optionFrame", UIParent , "UIPanelScrollFrameTemplate");
	mainFrame.optionsFrame.name = L["optionsConfig"];
	mainFrame.optionsFrame.parent = "RareCaller";
	mainFrame.optionsFrame.refresh = OptionsFrameSettingsLoad;
	mainFrame.optionsFrame.okay = OptionsFrameRefreshText;

	-- Respond to Request option
	mainFrame.optionsFrame.chkRespondToRequest = CreateCheckButton(mainFrame.optionsFrame, "chkRespondToRequest", 20, -20, (L["respond"]))
	mainFrame.optionsFrame.chkRespondToRequest.tooltip = (L["respondRT"])
	mainFrame.optionsFrame.chkRespondToRequest:SetScript("OnClick",
		function()
			if (mainFrame.optionsFrame.chkRespondToRequest:GetChecked()) then
				RareCallerDB.Options.RespondToRequests = true
			else
				RareCallerDB.Options.RespondToRequests = false
			end
		end
	);
		
	-- Report finds/death to General
	mainFrame.optionsFrame.chkReportFindsToGeneral = CreateCheckButton(mainFrame.optionsFrame, "chkReportFindsToGeneral", 20, -42, L["announce"])
	mainFrame.optionsFrame.chkReportFindsToGeneral.tooltip = (L["announce"])
	mainFrame.optionsFrame.chkReportFindsToGeneral:SetScript("OnClick",
		function()
			--print("mainFrame.optionsFrame.chkReportFindsToGeneral:GetChecked(): "..tostring(mainFrame.optionsFrame.chkReportFindsToGeneral:GetChecked()));
			if (mainFrame.optionsFrame.chkReportFindsToGeneral:GetChecked()) then
				--print('chkReportFindsToGeneral on');
				RareCallerDB.Options.ReportFindsToGeneral = true
			else
				--print('chkReportFindsToGeneral off');
				RareCallerDB.Options.ReportFindsToGeneral = false
			end
		end
	);

	-- Hide in combat
	mainFrame.optionsFrame.chkHideInCombat = CreateCheckButton(mainFrame.optionsFrame, "chkHideInCombat", 20, -64, (L["hidetracking"]))
	mainFrame.optionsFrame.chkHideInCombat.tooltip = (L["hidetracking"])
	mainFrame.optionsFrame.chkHideInCombat:SetScript("OnClick",
		function()
			if (mainFrame.optionsFrame.chkHideInCombat:GetChecked()) then
				RareCallerDB.Options.HideInCombat = true
			else
				RareCallerDB.Options.HideInCombat = false
			end
		end
	);

	-- Set TomTom Waypoints
	--mainFrame.optionsFrame.chkSetTomTom = CreateCheckButton(mainFrame.optionsFrame, "chkSetTomTom", 20, -86, L["setTomTom"])
	--mainFrame.optionsFrame.chkSetTomTom.tooltip = (L["setTomTom"])
	--mainFrame.optionsFrame.chkSetTomTom:SetScript("OnClick",
	--  function()
	--   if (mainFrame.optionsFrame.chkSetTomTom:GetChecked()) then
	--     RareCallerDB.Options.SetTomTomWaypoints = true
	--   else
	--     RareCallerDB.Options.SetTomTomWaypoints = false
	--   end
	--  end
	--);

	-- Tracking Type text
	mainFrame.optionsFrame.txtTrackOption = mainFrame.optionsFrame:CreateFontString("mainFrame.optionsFrame.txtTrackOption", "OVERLAY ", "GameFontNormal")
	mainFrame.optionsFrame.txtTrackOption:SetPoint("TOPLEFT", 24, -100)
	mainFrame.optionsFrame.txtTrackOption:SetText(L["trackTypeHead"])

    -- Tracking Type option Current Zone
    mainFrame.optionsFrame.rbnTrackCurrentZone = CreateRadioButton(mainFrame.optionsFrame, "rbnTrackCurrentZone", 30, -122, L["Current Zone"])
    mainFrame.optionsFrame.rbnTrackCurrentZone.tooltip = (L["trackTypeCurrentZoneTooltip"])
    mainFrame.optionsFrame.rbnTrackCurrentZone:SetScript("OnClick", SetTrackingType)
    -- Tracking Type option All
    mainFrame.optionsFrame.rbnTrackAll = CreateRadioButton(mainFrame.optionsFrame, "rbnTrackAll", 150, -122, L["All"])
    mainFrame.optionsFrame.rbnTrackAll.tooltip = (L["trackTypeAllTooltip"])
    mainFrame.optionsFrame.rbnTrackAll:SetScript("OnClick", SetTrackingType)

	InterfaceOptions_AddCategory(mainFrame.optionsFrame);
	--InterfaceOptions_AddCategory(mainFrame.trackOptionFrame);
-- === OPTION PANELS END === --

rareFrame:SetMovable(true)
rareFrame:EnableMouse(true)
rareFrame:SetUserPlaced(true)
rareFrame:RegisterForDrag("LeftButton")
rareFrame:SetScript("OnDragStart", DragStart)
rareFrame:SetScript("OnDragStop", DragStop)

mainFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
mainFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
mainFrame:RegisterEvent("CHANNEL_ROSTER_UPDATE")
mainFrame:RegisterEvent("ADDON_LOADED")
mainFrame:SetScript("OnEvent", MainEvents)

mainFrame.updateFrame = CreateFrame("Frame", "mainFrame.updateFrame", mainFrame)
mainFrame.updateFrame:SetScript("OnUpdate", OverallUpdater)
OverallUpdater(mainFrame, 10)

function addon:FakeNotice(health)
	health = tonumber(health) or 100;
	if health < 0 then 
		addon:Printf("Heath (%d) is less than zero. Resetting to 0", health);
		health = 0;
		--addon:Printf("Health: %d", health);
	elseif health > 100 then
		addon:Printf("Heath (%d) is over 100. Resetting to 100", health);
		health = 100;
		--addon:Printf("Health: %d", health);
	end;

	--[[
		npcID: 103
		npcName: Garrick Padfoot
		coords: 57.5,48.2
		mapAreaID: 30 --Elwynn Forest
		lastSeen: 
		
		[RCALLER]{npcID}_s_{time}_{healthPercentage}_{coords}_{mapAreaID}_{npcName}

		npcID: 103
		npcName: Garrick Padfoot
		coords: 57.5,48.2
		mapAreaID: 30 --Elwynn Forest
		lastSeen: 			

	--]]

	local npcID = 103; --Garrick Padfoot
	local coords = { 57.5, 48.2 };
	local mapAreaID = 30;
	local npcName = "Garrick Padfoot";
	
	--[RCALLER]{npcID}_s_{time}_{healthPercentage}_{coords}_{mapAreaID}_{npcName}
	local message = string.format("[%s]%d_s_%d_%d_%d_%d_%s",
			syncChanNamePrefix,
			npcID,
			time(),
			health,
			coords[1]*10000+coords[2]*10,
			mapAreaID,
			npcName);
			
	ProcessSyncChatMsg(message);
end;

function addon:ClearRares(npcToDelete)

	if npcToDelete == nil then		
		addon:Printf("Clear rares: specify either mob name, id, or \"all\"");
		return;
	end;
	
	npcToDelete = string.lower(npcToDelete);
		
	if (npcToDelete == "all") then
		RareCallerDB.Mobs = {};
		RefreshText("ClearRares(all)");
		return;
	end;

	local sNpcID, mobInfo;
	for sNpcID, mobInfo in pairs(RareCallerDB.Mobs) do
		if SameText(sNpcID, npcToDelete) or (string.find(string.lower(mobInfo.npcName), npcToDelete) ~= nil) then
			addon:Printf("Deleteing NPC %s (%s) from internal database", mobInfo.npcName, sNpcID);
			RareCallerDB.Mobs[sNpcID] = nil;
		end
	end
end;

function addon:ShouldTrack(npcID)
	npcID = tonumber(npcID);
	
	local bShouldTrack, reason = ShouldTrackRare(npcID);
	addon:Printf("Should track %d: %s - %s", npcID, tostring(bShouldTrack), tostring(reason));
end;

--[[
	RareCallerCommand

		--> SLASH_RareCallerCommand[n]
				SLASH_RareCallerCommand1 = "/rarecaller"
				SLASH_RareCallerCommand2 = "/rc"
--]]
SLASH_RareCallerCommand1 = "/rarecaller";
SLASH_RareCallerCommand2 = "/rc";
SlashCmdList["RareCallerCommand"] = function(msg)

	local command, arg1, arg2, arg3 = strsplit(" ", msg);	

	if SameText(command, "update") then	
		RefreshText("SlashCommand");
		addon:Printf("Updated text");
	elseif SameText(command, "fakenotice") then
		addon:FakeNotice(arg1);
	elseif SameText(command, "list") then
		addon:ListRares(arg1);
	elseif SameText(command, "toggle") or SameText(command, "announce") or SameText(command, "ann") then
		addon:ToggleAnnouncements();
	elseif SameText(command, "remove") then
		addon:ClearRares(arg1);
	elseif SameText(command, "reset") then
		RareCallerDB = {};
		RareCallerDB.Mobs = {};
		LoadSavedVariables();
	elseif SameText(command, "raretimer") or SameText(command, "raretime") then	
		addon:RareTimer(arg1);
	elseif SameText(command, "selftest") then	
		addon:SelfTest();
	elseif SameText(command, "shouldtrack") then
		addon:ShouldTrack(arg1);
	elseif SameText(command, "debuglevel") then
		--addon:Printf("arg1: \"%s\"", tostring(arg1));
		local level = tonumber(arg1);
		if level == nil then
			level = addon.debugLevel or 0;
			local s;
			if level == Level.Critical then
				s = "Critical";
			elseif level == Level.Error then
				s = "Error"
			elseif level == Level.Warning then
				s = "Warning"
			elseif level == Level.Information then	
				s = "Information";
			elseif level == Level.Verbose then
				s = "Verbose"
			elseif level == Level.None then		
				s = "None"
			else
				s = "";
			end;
			addon:Printf("Current debug level: %d (%s)", tostring(addon.debugLevel or 0), tostring(s));
		else
			addon.debugLevel = level;
			addon:Printf("DebugLevel set to %d", addon.debugLevel);
		end;
	elseif SameText(command, "debug") then
		if (addon.debugLevel == Level.Verbose) then
			addon.debugLevel = Level.Warning;
			addon:Printf("Reset logging to warning");
		else
			addon.debugLevel = Level.Verbose;
			addon:Printf("Verbose debug logging enabled");
		end;
	elseif SameText(command, "") then		
		--Print help
		addon:Printf(L["RareCaller slash commands"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "toggle", L["SlashCommand_toggle"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "update", L["SlashCommand_update"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "rareTimer name|npcID", L["SlashCommand_rareTimer"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "list [name|npcID]", L["SlashCommand_list"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "remove name|npcID|all", L["SlashCommand_remove"]); 
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "shouldTrack npcID", L["SlashCommand_shouldTrack"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "debugLevel [n]", L["SlashCommand_debugLevel"]);
		addon:Printf(YELLOW_FONT_COLOR_CODE.."%s"..FONT_COLOR_CODE_CLOSE.." - %s", "debug", L["SlashCommand_debug"]); 
		
	else	
		addon:Printf("Unknown command: \"%s\"", msg);
	end;
	--DEFAULT_CHAT_FRAME:AddMessage(msg or 'nil')
end;